/*
 * <sqlibf.h>
 */

#include <curl/curl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>


// Macros
/////////////////////////////////////////////////////////////////////////////

#define IMPRIME(f, s...) if(!options.quick_mode) printf(f, ## s);


// Defines
/////////////////////////////////////////////////////////////////////////////

#define TIMEOUT       120
#define MAX_FAILS     2
#define STRING_SIZE   4096
#define LIST_SIZE     1000
#define WORD_SIZE     64
#define BLIND_MAX     40
#define UNION_MAX     120


// Injection types

#define UNESCAPED           1     // and 1=1
#define SINGLE_QUOTE        2     // '
#define DOUBLE_QUOTES       3     // "
#define NUMERIC             4     // -0
#define STRING_CONCAT_PIPE  5     // || -> Oracle, DB2, postgres, informix, sqlite
#define STRING_CONCAT_PLUS  6     // + -> Mysql?, MSSQL, Sybase, access
#define SELECT_CASE_NUMERIC 7     // -(select case when 1=1 then 1/0 else 0 end)
#define SELECT_CASE_STRING  8     // || o + (select case when 1=1 then char(22222222222222) else '' end)
#define TIME_DELAY          9

// DDBB types

#define MSSQL         1
#define ORACLE        2
#define DB2           3
#define MYSQL         4
#define POSTGRESQL    5
#define INFORMIX      6
#define SYBASE        7
#define ACCESS        8
#define POINTBASE     9
#define SQLITE        10
#define HSQLDB        11
#define INGRES        12
#define INTERBASE     13
#define MYSQL4        14
#define MYSQL5        15

// Terminator types

#define GG            1     // '--'
#define BA            2     // '/*'
#define PC            3     // ';' (%3B)
#define NL            4     // '%00'
#define HA            5     // '#' (%23)
#define P1            6     // ')--' (%23)
#define P2            7     // '))--' (%23)
#define FINGER        10    // DDBB fingerprint

// Test types

#define KEY_WORD      0
#define LINES         1
#define WORDS         2

// Dinamic Type

#define STATIC        0
#define DINAMIC_LW    1
#define DINAMIC_KEY   2



#define SEPARATOR     "&"


// Globales
/////////////////////////////////////////////////////////////////////////////

CURL *curl;

struct result base_result;
struct result base_error;

int errores;
int analiza_html;
int db_from_finger;
int db_from_error;
int do_post;
int doing_union;

char key_word[WORD_SIZE];


// Estructuras
/////////////////////////////////////////////////////////////////////////////

// Opciones

struct opciones {
  char header[STRING_SIZE];
  int test_type;
  int uri_encode_data;
  int debuging;
  int quick_mode;
  int parameter;
  int injection;
  int database;
  } options;

// Resultado de una peticion

struct result {
  char url[STRING_SIZE];
  char post[STRING_SIZE];
  CURLcode estado;
  int codigo_http;
  int head_size;
  int head_lines;
  int body_size;
  int body_words;
  int body_lines;
  int word_count;
  char location[STRING_SIZE];
  char server[STRING_SIZE];
  char word_list[LIST_SIZE][WORD_SIZE];
};

// Parametros

struct query_string {
  struct query_string *anterior;
  struct query_string *siguiente;
  char *variable;
  char *valor;
  int is_dinamic;
  int is_injectable;
  int parentesis;
  int terminator;
  int error_words;
  int error_lines;
  char my_keyword[WORD_SIZE];
  };

// URLs

struct cracked_url {
  char *base;
  int param_num;
  int db_version;
  struct query_string *url_params;
  struct query_string *post_params;
  };

struct assembled_url {
  char *url;
  char *post;
  };

struct array {
  int code;
  char string[STRING_SIZE];
  };


// Funciones
/////////////////////////////////////////////////////////////////////////////

size_t get_body(void *ptr, size_t size, size_t nmemb, void *stream);
size_t get_header(void *ptr, size_t size, size_t nmemb, void *stream);
struct result get_url(struct assembled_url au);
int crack_body();

struct cracked_url crack_url(char *url, char *post);
struct assembled_url assemble_url(struct cracked_url cu, int num, char *append);
char *variable_name(struct cracked_url cu, int num);

int test_dinamic(struct cracked_url cu, int param);
int isdinamic(struct cracked_url cu, int num);

int test_inject(struct cracked_url cu);
int isinject(struct cracked_url cu, int num);

int fingerprint_bbdd(struct cracked_url cu);

int extra(struct cracked_url cu);
int check_parentesis(struct cracked_url cu, int num);
int check_terminator(struct cracked_url cu, int num);

int check_union(struct cracked_url cu);
int confirm_union(int campos, struct cracked_url cu, int counter, char *append);

int search(char *string);
int match(char *string, char *pattern);

char *code2string(struct array *a, int v);
char *uri_encode(char *string);
int check_url(char *url);
int compare_words(char list1[LIST_SIZE][WORD_SIZE], char list2[LIST_SIZE][WORD_SIZE], int count1, int count2);
char *get_diferent(char list1[LIST_SIZE][WORD_SIZE], char list2[LIST_SIZE][WORD_SIZE], int count1, int count2);
int is_inlist(char word[WORD_SIZE], char list1[LIST_SIZE][WORD_SIZE], int count);
int spaces_count(char *cadena);

int check_blind(struct cracked_url cu, char *my_query);


// SQL
///////////////////////////////////////////////////////////////////////////////

struct strings
{
    int database;
    char user[100];
    char version[100];
    char databases[200];
    char tables[200];
    char columns[200];
};








